@@ -48,10 +48,10 @@ __SYSCALL(__NR_rt_sigreturn, stub_rt_sigreturn)
 
 #define __NR_ioctl                              16
 __SYSCALL(__NR_ioctl, sys_ioctl)
-#define __NR_pread                              17
-__SYSCALL(__NR_pread, sys_pread)
-#define __NR_pwrite                             18
-__SYSCALL(__NR_pwrite, sys_pwrite)
+#define __NR_pread64                            17
+__SYSCALL(__NR_pread64, sys_pread64)
+#define __NR_pwrite64                           18
+__SYSCALL(__NR_pwrite64, sys_pwrite64)
 #define __NR_readv                              19
 __SYSCALL(__NR_readv, sys_readv)
 #define __NR_writev                             20
@@ -98,7 +98,7 @@ __SYSCALL(__NR_setitimer, sys_setitimer)
 __SYSCALL(__NR_getpid, sys_getpid)
 
 #define __NR_sendfile                           40
-__SYSCALL(__NR_sendfile, sys_sendfile)
+__SYSCALL(__NR_sendfile, sys_sendfile64)
 #define __NR_socket                             41
 __SYSCALL(__NR_socket, sys_socket)
 #define __NR_connect                            42
@@ -399,15 +399,15 @@ __SYSCALL(__NR_iopl, stub_iopl)
 __SYSCALL(__NR_ioperm, sys_ioperm)
 
 #define __NR_create_module                     174
-__SYSCALL(__NR_create_module, sys_create_module)
+__SYSCALL(__NR_create_module, sys_ni_syscall)
 #define __NR_init_module                       175
 __SYSCALL(__NR_init_module, sys_init_module)
 #define __NR_delete_module                     176
 __SYSCALL(__NR_delete_module, sys_delete_module)
 #define __NR_get_kernel_syms                   177
-__SYSCALL(__NR_get_kernel_syms, sys_get_kernel_syms)
+__SYSCALL(__NR_get_kernel_syms, sys_ni_syscall)
 #define __NR_query_module                      178
-__SYSCALL(__NR_query_module, sys_query_module)
+__SYSCALL(__NR_query_module, sys_ni_syscall)
 
 #define __NR_quotactl                          179
 __SYSCALL(__NR_quotactl, sys_quotactl)
@@ -415,18 +415,18 @@ __SYSCALL(__NR_quotactl, sys_quotactl)
 #define __NR_nfsservctl                        180
 __SYSCALL(__NR_nfsservctl, sys_nfsservctl)
 
-#define __NR_getpmsg                           181
+#define __NR_getpmsg                           181	/* reserved for LiS/STREAMS */
 __SYSCALL(__NR_getpmsg, sys_ni_syscall)
-#define __NR_putpmsg                           182
+#define __NR_putpmsg                           182	/* reserved for LiS/STREAMS */
 __SYSCALL(__NR_putpmsg, sys_ni_syscall)
 
-#define __NR_afs_syscall                       183
+#define __NR_afs_syscall                       183	/* reserved for AFS */ 
 __SYSCALL(__NR_afs_syscall, sys_ni_syscall)
 
 #define __NR_tuxcall      		184 /* reserved for tux */
 __SYSCALL(__NR_tuxcall, sys_ni_syscall)
 
-#define __NR_security			185 /* reserved for security */
+#define __NR_security			185
 __SYSCALL(__NR_security, sys_ni_syscall)
 
 #define __NR_gettid		186
@@ -434,76 +434,134 @@ __SYSCALL(__NR_gettid, sys_gettid)
 
 #define __NR_readahead		187
 __SYSCALL(__NR_readahead, sys_readahead)
-
 #define __NR_setxattr		188
-__SYSCALL(__NR_setxattr, sys_ni_syscall)
+__SYSCALL(__NR_setxattr, sys_setxattr)
 #define __NR_lsetxattr		189
-__SYSCALL(__NR_lsetxattr, sys_ni_syscall)
+__SYSCALL(__NR_lsetxattr, sys_lsetxattr)
 #define __NR_fsetxattr		190
-__SYSCALL(__NR_fsetxattr, sys_ni_syscall)
+__SYSCALL(__NR_fsetxattr, sys_fsetxattr)
 #define __NR_getxattr		191
-__SYSCALL(__NR_getxattr, sys_ni_syscall)
+__SYSCALL(__NR_getxattr, sys_getxattr)
 #define __NR_lgetxattr		192
-__SYSCALL(__NR_lgetxattr, sys_ni_syscall)
+__SYSCALL(__NR_lgetxattr, sys_lgetxattr)
 #define __NR_fgetxattr		193
-__SYSCALL(__NR_fgetxattr, sys_ni_syscall) 
+__SYSCALL(__NR_fgetxattr, sys_fgetxattr) 
 #define __NR_listxattr		194
-__SYSCALL(__NR_listxattr, sys_ni_syscall) 
+__SYSCALL(__NR_listxattr, sys_listxattr) 
 #define __NR_llistxattr		195
-__SYSCALL(__NR_llistxattr, sys_ni_syscall) 
+__SYSCALL(__NR_llistxattr, sys_llistxattr) 
 #define __NR_flistxattr		196
-__SYSCALL(__NR_flistxattr, sys_ni_syscall) 
+__SYSCALL(__NR_flistxattr, sys_flistxattr) 
 #define __NR_removexattr	197
-__SYSCALL(__NR_removexattr, sys_ni_syscall) 
+__SYSCALL(__NR_removexattr, sys_removexattr) 
 #define __NR_lremovexattr	198
-__SYSCALL(__NR_lremovexattr, sys_ni_syscall) 
+__SYSCALL(__NR_lremovexattr, sys_lremovexattr) 
 #define __NR_fremovexattr	199
-__SYSCALL(__NR_fremovexattr, sys_ni_syscall) 
-#define __NR_tkill	200	/* 2.5 only */
-__SYSCALL(__NR_tkill, sys_ni_syscall) 
-#define __NR_time		201
+__SYSCALL(__NR_fremovexattr, sys_fremovexattr) 
+#define __NR_tkill	200
+__SYSCALL(__NR_tkill, sys_tkill) 
+#define __NR_time      201
 __SYSCALL(__NR_time, sys_time64)
-#define __NR_futex		202 /* 2.5 only */
-__SYSCALL(__NR_futex, sys_ni_syscall)
+#define __NR_futex     202
+__SYSCALL(__NR_futex, sys_futex)
 #define __NR_sched_setaffinity    203
-__SYSCALL(__NR_sched_setaffinity, sys_ni_syscall)
+__SYSCALL(__NR_sched_setaffinity, sys_sched_setaffinity)
 #define __NR_sched_getaffinity     204
-__SYSCALL(__NR_sched_getaffinity, sys_ni_syscall)
+__SYSCALL(__NR_sched_getaffinity, sys_sched_getaffinity)
 #define __NR_set_thread_area	205
-__SYSCALL(__NR_set_thread_area, sys_ni_syscall)
+__SYSCALL(__NR_set_thread_area, sys_ni_syscall)	/* use arch_prctl */
 #define __NR_io_setup	206
-__SYSCALL(__NR_io_setup, sys_ni_syscall)
+__SYSCALL(__NR_io_setup, sys_io_setup)
 #define __NR_io_destroy	207
-__SYSCALL(__NR_io_destroy, sys_ni_syscall)
+__SYSCALL(__NR_io_destroy, sys_io_destroy)
 #define __NR_io_getevents	208
-__SYSCALL(__NR_io_getevents, sys_ni_syscall)
+__SYSCALL(__NR_io_getevents, sys_io_getevents)
 #define __NR_io_submit	209
-__SYSCALL(__NR_io_submit, sys_ni_syscall)
+__SYSCALL(__NR_io_submit, sys_io_submit)
 #define __NR_io_cancel	210
-__SYSCALL(__NR_io_cancel, sys_ni_syscall)
+__SYSCALL(__NR_io_cancel, sys_io_cancel)
 #define __NR_get_thread_area	211
-__SYSCALL(__NR_get_thread_area, sys_ni_syscall)
-#define __NR_lookup_dcookie     212
-__SYSCALL(__NR_lookup_dcookie, sys_ni_syscall)
-#define __NR_epoll_create       213
-__SYSCALL(__NR_epoll_create, sys_ni_syscall)
-#define __NR_epoll_ctl  214
-__SYSCALL(__NR_epoll_ctl, sys_ni_syscall)
-#define __NR_epoll_wait 215
-__SYSCALL(__NR_epoll_wait, sys_ni_syscall)
-#define __NR_remap_file_pages   216
-__SYSCALL(__NR_remap_file_pages, sys_ni_syscall)
-#define __NR_getdents64 217
+__SYSCALL(__NR_get_thread_area, sys_ni_syscall)	/* use arch_prctl */
+#define __NR_lookup_dcookie	212
+__SYSCALL(__NR_lookup_dcookie, sys_lookup_dcookie)
+#define __NR_epoll_create	213
+__SYSCALL(__NR_epoll_create, sys_epoll_create)
+#define __NR_epoll_ctl_old	214
+__SYSCALL(__NR_epoll_ctl_old, sys_ni_syscall)
+#define __NR_epoll_wait_old	215
+__SYSCALL(__NR_epoll_wait_old, sys_ni_syscall)
+#define __NR_remap_file_pages	216
+__SYSCALL(__NR_remap_file_pages, sys_remap_file_pages)
+#define __NR_getdents64	217
 __SYSCALL(__NR_getdents64, sys_getdents64)
+#define __NR_set_tid_address	218
+__SYSCALL(__NR_set_tid_address, sys_set_tid_address)
 #define __NR_restart_syscall	219
-__SYSCALL(__NR_restart_syscall, sys_ni_syscall)
+__SYSCALL(__NR_restart_syscall, sys_restart_syscall)
 #define __NR_semtimedop		220
 __SYSCALL(__NR_semtimedop, sys_semtimedop)
+#define __NR_fadvise64		221
+__SYSCALL(__NR_fadvise64, sys_fadvise64)
+#define __NR_timer_create		222
+__SYSCALL(__NR_timer_create, sys_timer_create)
+#define __NR_timer_settime		223
+__SYSCALL(__NR_timer_settime, sys_timer_settime)
+#define __NR_timer_gettime		224
+__SYSCALL(__NR_timer_gettime, sys_timer_gettime)
+#define __NR_timer_getoverrun		225
+__SYSCALL(__NR_timer_getoverrun, sys_timer_getoverrun)
+#define __NR_timer_delete	226
+__SYSCALL(__NR_timer_delete, sys_timer_delete)
+#define __NR_clock_settime	227
+__SYSCALL(__NR_clock_settime, sys_clock_settime)
+#define __NR_clock_gettime	228
+__SYSCALL(__NR_clock_gettime, sys_clock_gettime)
+#define __NR_clock_getres	229
+__SYSCALL(__NR_clock_getres, sys_clock_getres)
+#define __NR_clock_nanosleep	230
+__SYSCALL(__NR_clock_nanosleep, sys_clock_nanosleep)
+#define __NR_exit_group		231
+__SYSCALL(__NR_exit_group, sys_exit_group)
+#define __NR_epoll_wait		232
+__SYSCALL(__NR_epoll_wait, sys_epoll_wait)
+#define __NR_epoll_ctl		233
+__SYSCALL(__NR_epoll_ctl, sys_epoll_ctl)
+#define __NR_tgkill		234
+__SYSCALL(__NR_tgkill, sys_tgkill)
+#define __NR_utimes		235
+__SYSCALL(__NR_utimes, sys_utimes)
+#define __NR_vserver		236
+__SYSCALL(__NR_vserver, sys_ni_syscall)
+#define __NR_vserver		236
+__SYSCALL(__NR_vserver, sys_ni_syscall)
+#define __NR_mbind 		237
+__SYSCALL(__NR_mbind, sys_mbind)
+#define __NR_set_mempolicy 	238
+__SYSCALL(__NR_set_mempolicy, sys_set_mempolicy)
+#define __NR_get_mempolicy 	239
+__SYSCALL(__NR_get_mempolicy, sys_get_mempolicy)
+#define __NR_mq_open 		240
+__SYSCALL(__NR_mq_open, sys_mq_open)
+#define __NR_mq_unlink 		241
+__SYSCALL(__NR_mq_unlink, sys_mq_unlink)
+#define __NR_mq_timedsend 	242
+__SYSCALL(__NR_mq_timedsend, sys_mq_timedsend)
+#define __NR_mq_timedreceive	243
+__SYSCALL(__NR_mq_timedreceive, sys_mq_timedreceive)
+#define __NR_mq_notify 		244
+__SYSCALL(__NR_mq_notify, sys_mq_notify)
+#define __NR_mq_getsetattr 	245
+__SYSCALL(__NR_mq_getsetattr, sys_mq_getsetattr)
+#define __NR_kexec_load 	246
+__SYSCALL(__NR_kexec_load, sys_ni_syscall)
+#define __NR_waitid		247
+__SYSCALL(__NR_waitid, sys_waitid)
 
-#define __NR_syscall_max __NR_semtimedop
-
+#define __NR_syscall_max __NR_waitid
 #ifndef __NO_STUBS
 
+/* user-visible error numbers are in the range -1 - -4095 */
+
 #define __syscall_clobber "r11","rcx","memory" 
 
 #define __syscall_return(type, res) \
@@ -515,11 +573,8 @@ do { \
 	return (type) (res); \
 } while (0)
 
-#ifndef __KERNEL_SYSCALLS__
-
 #define __syscall "syscall"
 
-
 #define _syscall0(type,name) \
 type name(void) \
 { \
@@ -565,10 +620,10 @@ __syscall_return(type,__res); \
 type name (type1 arg1, type2 arg2, type3 arg3, type4 arg4) \
 { \
 long __res; \
-__asm__ volatile ("movq %[a3],%%r10 ;" __syscall \
+__asm__ volatile ("movq %5,%%r10 ;" __syscall \
 	: "=a" (__res) \
 	: "0" (__NR_##name),"D" ((long)(arg1)),"S" ((long)(arg2)), \
-	  "d" ((long)(arg3)), [a3] "g" ((long)(arg4)) : __syscall_clobber,"r10" ); \
+	  "d" ((long)(arg3)),"g" ((long)(arg4)) : __syscall_clobber,"r10" ); \
 __syscall_return(type,__res); \
 } 
 
@@ -577,10 +632,10 @@ __syscall_return(type,__res); \
 type name (type1 arg1,type2 arg2,type3 arg3,type4 arg4,type5 arg5) \
 { \
 long __res; \
-__asm__ volatile ("movq %[a3],%%r10 ; movq %[a4],%%r8 ; " __syscall \
+__asm__ volatile ("movq %5,%%r10 ; movq %6,%%r8 ; " __syscall \
 	: "=a" (__res) \
 	: "0" (__NR_##name),"D" ((long)(arg1)),"S" ((long)(arg2)), \
-	  "d" ((long)(arg3)), [a3] "g" ((long)(arg4)), [a4] "g" ((long)(arg5)) : \
+	  "d" ((long)(arg3)),"g" ((long)(arg4)),"g" ((long)(arg5)) : \
 	__syscall_clobber,"r8","r10" ); \
 __syscall_return(type,__res); \
 }
@@ -590,113 +645,23 @@ __syscall_return(type,__res); \
 type name (type1 arg1,type2 arg2,type3 arg3,type4 arg4,type5 arg5,type6 arg6) \
 { \
 long __res; \
-__asm__ volatile ("movq %[a3],%%r10 ; movq %[a4],%%r8 ; movq %[a5],%%r9 ; " __syscall \
+__asm__ volatile ("movq %5,%%r10 ; movq %6,%%r8 ; movq %7,%%r9 ; " __syscall \
 	: "=a" (__res) \
 	: "0" (__NR_##name),"D" ((long)(arg1)),"S" ((long)(arg2)), \
-	  "d" ((long)(arg3)), [a3] "g" ((long)(arg4)), [a4] "g" ((long)(arg5)), \
-	  [a5] "g" ((long)(arg6)) : \
+	  "d" ((long)(arg3)), "g" ((long)(arg4)), "g" ((long)(arg5)), \
+	  "g" ((long)(arg6)) : \
 	__syscall_clobber,"r8","r10","r9" ); \
 __syscall_return(type,__res); \
 }
 
-#else /* __KERNEL_SYSCALLS__ */
-
-/*
- * we need this inline - forking from kernel space will result
- * in NO COPY ON WRITE (!!!), until an execve is executed. This
- * is no problem, but for the stack. This is handled by not letting
- * main() use the stack at all after fork(). Thus, no function
- * calls - which means inline code for fork too, as otherwise we
- * would use the stack upon exit from 'fork()'.
- *
- * Actually only pause and fork are needed inline, so that there
- * won't be any messing with the stack from main(), but we define
- * some others too.
- */
-#define __NR__exit __NR_exit
-
-extern long sys_pause(void);
-static inline long pause(void)
-{
-	return sys_pause();
-}
-
-extern long sys_sync(void);
-static inline long sync(void)
-{
-	return sys_sync();
-}
-
-extern pid_t sys_setsid(void);
-static inline pid_t setsid(void)
-{
-	return sys_setsid();
-}
-
-extern ssize_t sys_write(unsigned int, char *, size_t);
-static inline ssize_t write(unsigned int fd, char * buf, size_t count)
-{
-	return sys_write(fd, buf, count);
-}
-
-extern ssize_t sys_read(unsigned int, char *, size_t);
-static inline ssize_t read(unsigned int fd, char * buf, size_t count)
-{
-	return sys_read(fd, buf, count);
-}
-
-extern off_t sys_lseek(unsigned int, off_t, unsigned int);
-static inline off_t lseek(unsigned int fd, off_t offset, unsigned int origin)
-{
-	return sys_lseek(fd, offset, origin);
-}
-
-extern long sys_dup(unsigned int);
-static inline long dup(unsigned int fd)
-{
-	return sys_dup(fd);
-}
-
-/* implemented in asm in arch/x86_64/kernel/entry.S */
-extern long execve(char *, char **, char **);
-
-extern long sys_open(const char *, int, int);
-static inline long open(const char * filename, int flags, int mode)
-{
-	return sys_open(filename, flags, mode);
-}
-
-extern long sys_close(unsigned int);
-static inline long close(unsigned int fd)
-{
-	return sys_close(fd);
-}
-
-extern long sys_exit(int) __attribute__((noreturn));
-static inline void exit(int error_code)
-{
-	sys_exit(error_code);
-}
-
-extern long sys_delete_module(const char *);
-static inline long delete_module(const char *name_user)
-{
-	return sys_delete_module(name_user);
-}
-
-extern long sys_wait4(pid_t, unsigned int *, int, struct rusage *);
-static inline pid_t waitpid(int pid, int * wait_stat, int flags)
-{
-	return sys_wait4(pid, wait_stat, flags, NULL);
-}
-
-static inline pid_t wait(int * wait_stat)
-{
-	return waitpid(-1,wait_stat,0);
-}
-
-#endif /* __KERNEL_SYSCALLS__ */
-
 #endif /* __NO_STUBS */
 
+/*
+ * "Conditional" syscalls
+ *
+ * What we want is __attribute__((weak,alias("sys_ni_syscall"))),
+ * but it doesn't work on all toolchains, so we just do it by hand
+ */
+#define cond_syscall(x) asm(".weak\t" #x "\n\t.set\t" #x ",sys_ni_syscall");
+
 #endif
