@@ -1,5 +1,3 @@
-/* $USAGI: sysctl.h,v 1.36 2004/01/11 08:19:25 yoshfuji Exp $ */
-
 /*
  * sysctl.h: General linux system control interface
  *
@@ -8,16 +6,10 @@
  ****************************************************************
  ****************************************************************
  **
- **  WARNING:  
  **  The values in this file are exported to user space via 
- **  the sysctl() binary interface.  Do *NOT* change the 
- **  numbering of any existing values here, and do not change
- **  any numbers within any one set of values.  If you have
- **  to redefine an existing interface, use a new number for it.
- **  The kernel will then return ENOTDIR to any application using
- **  the old binary interface.
- **
- **  --sct
+ **  the sysctl() binary interface.  However this interface
+ **  is unstable and deprecated and will be removed in the future. 
+ **  For a stable interface use /proc/sys.
  **
  ****************************************************************
  ****************************************************************
@@ -26,13 +18,12 @@
 #ifndef _LINUX_SYSCTL_H
 #define _LINUX_SYSCTL_H
 
-#include <linux/kernel.h>
 #include <linux/types.h>
-#include <linux/list.h>
 
-struct file;
-
-#define CTL_MAXNAME 10
+#define CTL_MAXNAME 10		/* how many path components do we allow in a
+				   call to sysctl?   In other words, what is
+				   the largest acceptable value for the nlen
+				   member of a struct __sysctl_args to have? */
 
 struct __sysctl_args {
 	int *name;
@@ -48,12 +39,6 @@ struct __sysctl_args {
 
 /* Top-level names: */
 
-/* For internal pattern-matching use only: */
-#ifdef __KERNEL__
-#define CTL_ANY		-1	/* Matches any name */
-#define CTL_NONE	0
-#endif
-
 enum
 {
 	CTL_KERN=1,		/* General kernel info and control */
@@ -126,40 +111,63 @@ enum
 	KERN_CORE_USES_PID=52,		/* int: use core or core.%pid */
 	KERN_TAINTED=53,	/* int: various kernel tainted flags */
 	KERN_CADPID=54,		/* int: PID of the process to notify on CAD */
- 	KERN_CORE_PATTERN=56,	/* string: pattern for core-files */
-	KERN_PPC_L3CR=57,       /* l3cr register on PPC */
-	KERN_EXCEPTION_TRACE=58, /* boolean: exception trace */
- 	KERN_CORE_SETUID=59,	/* int: set to allow core dumps of setuid apps */
+	KERN_PIDMAX=55,		/* int: PID # limit */
+  	KERN_CORE_PATTERN=56,	/* string: pattern for core-file names */
+	KERN_PANIC_ON_OOPS=57,  /* int: whether we will panic on an oops */
+	KERN_HPPA_PWRSW=58,	/* int: hppa soft-power enable */
+	KERN_HPPA_UNALIGNED=59,	/* int: hppa unaligned-trap enable */
+	KERN_PRINTK_RATELIMIT=60, /* int: tune printk ratelimiting */
+	KERN_PRINTK_RATELIMIT_BURST=61,	/* int: tune printk ratelimiting */
+	KERN_PTY=62,		/* dir: pty driver */
+	KERN_NGROUPS_MAX=63,	/* int: NGROUPS_MAX */
 	KERN_SPARC_SCONS_PWROFF=64, /* int: serial console power-off halt */
+	KERN_HZ_TIMER=65,	/* int: hz timer on or off */
+	KERN_UNKNOWN_NMI_PANIC=66 /* int: unknown nmi panic flag */
 };
 
 
 /* CTL_VM names: */
 enum
 {
-	VM_SWAPCTL=1,		/* struct: Set vm swapping control */
-	VM_SWAPOUT=2,		/* int: Linear or sqrt() swapout for hogs */
-	VM_FREEPG=3,		/* struct: Set free page thresholds */
-	VM_BDFLUSH=4,		/* struct: Control buffer cache flushing */
+	VM_SWAPCTL=1,		/* [<2.6 ONLY] struct: Set vm swapping control */
+	VM_SWAPOUT=2,		/* [<2.6 ONLY] int: Linear or sqrt() swapout for hogs */
+	VM_FREEPG=3,		/* [<2.6 ONLY] struct: Set free page thresholds */
+	VM_BDFLUSH=4,		/* [<2.6 ONLY] struct: Control buffer cache flushing */
+	VM_UNUSED1=1,		/* was: struct: Set vm swapping control */
+	VM_UNUSED2=2,		/* was; int: Linear or sqrt() swapout for hogs */
+	VM_UNUSED3=3,		/* was: struct: Set free page thresholds */
+	VM_UNUSED4=4,		/* Spare */
 	VM_OVERCOMMIT_MEMORY=5,	/* Turn off the virtual memory safety limit */
-	VM_BUFFERMEM=6,		/* struct: Set buffer memory thresholds */
-	VM_PAGECACHE=7,		/* struct: Set cache memory thresholds */
-	VM_PAGERDAEMON=8,	/* struct: Control kswapd behaviour */
-	VM_PGT_CACHE=9,		/* struct: Set page table cache parameters */
+	VM_BUFFERMEM=6,		/* [<2.6 ONLY] struct: Set buffer memory thresholds */
+	VM_PAGECACHE=7,		/* [<2.6 ONLY] struct: Set cache memory thresholds */
+	VM_PAGERDAEMON=8,	/* [<2.6 ONLY] struct: Control kswapd behaviour */
+	VM_PGT_CACHE=9,		/* [<2.6 ONLY] struct: Set page table cache parameters */
+	VM_UNUSED5=6,		/* was: struct: Set buffer memory thresholds */
+	VM_UNUSED7=7,		/* was: struct: Set cache memory thresholds */
+	VM_UNUSED8=8,		/* was: struct: Control kswapd behaviour */
+	VM_UNUSED9=9,		/* was: struct: Set page table cache parameters */
 	VM_PAGE_CLUSTER=10,	/* int: set number of pages to swap together */
-	VM_MAX_MAP_COUNT=11,	/* int: Maximum number of active map areas */
-	VM_MIN_READAHEAD=12,    /* Min file readahead */
-	VM_MAX_READAHEAD=13,    /* Max file readahead */
-	VM_VFS_SCAN_RATIO=14,   /* part of the inactive vfs lists to scan */
-	VM_LRU_BALANCE_RATIO=15,/* balance active and inactive caches */
-	VM_PASSES=16,           /* number of vm passes before failing */
+	/*VM_MAX_MAP_COUNT=11,	[<2.6] int: Maximum number of active map areas */
+	VM_MIN_READAHEAD=12,    /* [<2.6] Min file readahead */
+	VM_MAX_READAHEAD=13,    /* [<2.6] Max file readahead */
+	VM_DIRTY_BACKGROUND=11,	/* dirty_background_ratio */
+	VM_DIRTY_RATIO=12,	/* dirty_ratio */
+	VM_DIRTY_WB_CS=13,	/* dirty_writeback_centisecs */
+	VM_DIRTY_EXPIRE_CS=14,	/* dirty_expire_centisecs */
+	VM_NR_PDFLUSH_THREADS=15, /* nr_pdflush_threads */
+	VM_OVERCOMMIT_RATIO=16, /* percent of RAM to allow overcommit in */
 	VM_PAGEBUF=17,		/* struct: Control pagebuf parameters */
-	VM_GFP_DEBUG=18,        /* debug GFP failures */
-	VM_CACHE_SCAN_RATIO=19, /* part of the inactive cache list to scan */
-	VM_MAPPED_RATIO=20,	/* amount of unfreeable pages that triggers swapout */
-	VM_LAPTOP_MODE=21,	/* kernel in laptop flush mode */
-	VM_BLOCK_DUMP=22,	/* dump fs activity to log */
-	VM_ANON_LRU=23,		/* immediatly insert anon pages in the vm page lru */
+	VM_HUGETLB_PAGES=18,	/* int: Number of available Huge Pages */
+	VM_SWAPPINESS=19,	/* Tendency to steal mapped memory */
+	VM_LOWER_ZONE_PROTECTION=20,/* Amount of protection of lower zones */
+	VM_MIN_FREE_KBYTES=21,	/* Minimum free kilobytes to maintain */
+	VM_MAX_MAP_COUNT=22,	/* int: Maximum number of mmaps/address-space */
+	VM_LAPTOP_MODE=23,	/* vm laptop mode */
+	VM_BLOCK_DUMP=24,	/* block dump mode */
+	VM_HUGETLB_GROUP=25,	/* permitted hugetlb group */
+	VM_VFS_CACHE_PRESSURE=26, /* dcache/icache reclaim pressure */
+	VM_LEGACY_VA_LAYOUT=27, /* legacy/compatibility virtual address space layout */
+	VM_SWAP_TOKEN_TIMEOUT=28 /* default time for token time out */
 };
 
 
@@ -182,8 +190,8 @@ enum
 	NET_TR=14,
 	NET_DECNET=15,
 	NET_ECONET=16,
-	NET_KHTTPD=17,
-	NET_SCTP=18
+	NET_KHTTPD=17,				/* [<2.6] */
+	NET_SCTP=17 
 };
 
 /* /proc/sys/kernel/random */
@@ -197,6 +205,13 @@ enum
 	RANDOM_UUID=6
 };
 
+/* /proc/sys/kernel/pty */
+enum
+{
+	PTY_MAX=1,
+	PTY_NR=2
+};
+
 /* /proc/sys/bus/isa */
 enum
 {
@@ -225,7 +240,7 @@ enum
 	NET_CORE_LO_CONG=15,
 	NET_CORE_MOD_CONG=16,
 	NET_CORE_DEV_WEIGHT=17,
-	NET_CORE_SOMAXCONN=18,
+	NET_CORE_SOMAXCONN=18
 };
 
 /* /proc/sys/net/ethernet */
@@ -238,7 +253,7 @@ enum
 {
 	NET_UNIX_DESTROY_DELAY=1,
 	NET_UNIX_DELETE_DELAY=2,
-	NET_UNIX_MAX_DGRAM_QLEN=3,
+	NET_UNIX_MAX_DGRAM_QLEN=3
 };
 
 /* /proc/sys/net/ipv4 */
@@ -328,6 +343,7 @@ enum
 	NET_TCP_BIC_LOW_WINDOW=104,
 	NET_TCP_DEFAULT_WIN_SCALE=105,
 	NET_TCP_MODERATE_RCVBUF=106,
+	NET_TCP_TSO_WIN_DIVISOR=107
 };
 
 enum {
@@ -348,7 +364,7 @@ enum {
 	NET_IPV4_ROUTE_MTU_EXPIRES=15,
 	NET_IPV4_ROUTE_MIN_PMTU=16,
 	NET_IPV4_ROUTE_MIN_ADVMSS=17,
-	NET_IPV4_ROUTE_SECRET_INTERVAL=18,
+	NET_IPV4_ROUTE_SECRET_INTERVAL=18
 };
 
 enum
@@ -375,9 +391,11 @@ enum
 	NET_IPV4_CONF_TAG=12,
 	NET_IPV4_CONF_ARPFILTER=13,
 	NET_IPV4_CONF_MEDIUM_ID=14,
+	NET_IPV4_CONF_NOXFRM=15,
+	NET_IPV4_CONF_NOPOLICY=16,
 	NET_IPV4_CONF_FORCE_IGMP_VERSION=17,
 	NET_IPV4_CONF_ARP_ANNOUNCE=18,
-	NET_IPV4_CONF_ARP_IGNORE=19,
+	NET_IPV4_CONF_ARP_IGNORE=19
 };
 
 /* /proc/sys/net/ipv4/netfilter */
@@ -397,6 +415,19 @@ enum
 	NET_IPV4_NF_CONNTRACK_ICMP_TIMEOUT=12,
 	NET_IPV4_NF_CONNTRACK_GENERIC_TIMEOUT=13,
 	NET_IPV4_NF_CONNTRACK_BUCKETS=14,
+	NET_IPV4_NF_CONNTRACK_LOG_INVALID=15,
+	NET_IPV4_NF_CONNTRACK_TCP_TIMEOUT_MAX_RETRANS=16,
+	NET_IPV4_NF_CONNTRACK_TCP_LOOSE=17,
+	NET_IPV4_NF_CONNTRACK_TCP_BE_LIBERAL=18,
+	NET_IPV4_NF_CONNTRACK_TCP_MAX_RETRANS=19,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_CLOSED=20,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_COOKIE_WAIT=21,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_COOKIE_ECHOED=22,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_ESTABLISHED=23,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_SHUTDOWN_SENT=24,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_SHUTDOWN_RECD=25,
+ 	NET_IPV4_NF_CONNTRACK_SCTP_TIMEOUT_SHUTDOWN_ACK_SENT=26,
+	NET_IPV4_NF_CONNTRACK_COUNT=27
 };
  
 /* /proc/sys/net/ipv6 */
@@ -409,10 +440,8 @@ enum {
 	NET_IPV6_IP6FRAG_HIGH_THRESH=21,
 	NET_IPV6_IP6FRAG_LOW_THRESH=22,
 	NET_IPV6_IP6FRAG_TIME=23,
-	/* NET_IPV6_IP6FRAG_SECRET_INTERVAL=24, */
-	NET_IPV6_MLD_MAX_MSF=25,
-	NET_IPV6_BINDV6ONLY_RESTRICTION=26,	/* unofficial */
-	NET_IPV6_MOBILITY=27,			/* unofficial */
+	NET_IPV6_IP6FRAG_SECRET_INTERVAL=24,
+	NET_IPV6_MLD_MAX_MSF=25
 };
 
 enum {
@@ -443,8 +472,8 @@ enum {
 	NET_IPV6_TEMP_PREFERED_LFT=13,
 	NET_IPV6_REGEN_MAX_RETRY=14,
 	NET_IPV6_MAX_DESYNC_FACTOR=15,
-	NET_IPV6_MAX_ADDRESSES=16,	/* unofficial */
-	NET_IPV6_ACCEPT_NI,	/* unofficial */
+	NET_IPV6_MAX_ADDRESSES=16,
+	NET_IPV6_FORCE_MLD_VERSION=17
 };
 
 /* /proc/sys/net/ipv6/icmp */
@@ -566,40 +595,6 @@ enum {
 	NET_DECNET_DEBUG_LEVEL = 255
 };
 
-/* /proc/sys/net/sctp */
-enum {
-	NET_SCTP_RTO_INITIAL = 1,
-	NET_SCTP_RTO_MIN     = 2,
-	NET_SCTP_RTO_MAX     = 3,
-	NET_SCTP_RTO_ALPHA   = 4,
-	NET_SCTP_RTO_BETA    = 5,
-	NET_SCTP_VALID_COOKIE_LIFE       =  6,
-	NET_SCTP_ASSOCIATION_MAX_RETRANS =  7,
-	NET_SCTP_PATH_MAX_RETRANS        =  8,
-	NET_SCTP_MAX_INIT_RETRANSMITS    =  9,
-	NET_SCTP_HB_INTERVAL             = 10,
-	NET_SCTP_PRESERVE_ENABLE         = 11,
-	NET_SCTP_MAX_BURST               = 12,
-	NET_SCTP_ADDIP_ENABLE            = 13,
-	NET_SCTP_PRSCTP_ENABLE           = 14,
-};
-/* /proc/sys/net/khttpd/ */
-enum {
-	NET_KHTTPD_DOCROOT	= 1,
-	NET_KHTTPD_START	= 2,
-	NET_KHTTPD_STOP		= 3,
-	NET_KHTTPD_UNLOAD	= 4,
-	NET_KHTTPD_CLIENTPORT	= 5,
-	NET_KHTTPD_PERMREQ	= 6,
-	NET_KHTTPD_PERMFORBID	= 7,
-	NET_KHTTPD_LOGGING	= 8,
-	NET_KHTTPD_SERVERPORT	= 9,
-	NET_KHTTPD_DYNAMICSTRING= 10,
-	NET_KHTTPD_SLOPPYMIME   = 11,
-	NET_KHTTPD_THREADS	= 12,
-	NET_KHTTPD_MAXCONNECT	= 13
-};
-
 /* /proc/sys/net/decnet/conf/<dev> */
 enum {
 	NET_DECNET_CONF_LOOPBACK = -2,
@@ -623,6 +618,49 @@ enum {
 	NET_DECNET_CONF_DEV_STATE = 7
 };
 
+/* /proc/sys/net/khttpd/ [<2.6 ONLY] */
+enum {
+	NET_KHTTPD_DOCROOT	= 1,
+	NET_KHTTPD_START	= 2,
+	NET_KHTTPD_STOP		= 3,
+	NET_KHTTPD_UNLOAD	= 4,
+	NET_KHTTPD_CLIENTPORT	= 5,
+	NET_KHTTPD_PERMREQ	= 6,
+	NET_KHTTPD_PERMFORBID	= 7,
+	NET_KHTTPD_LOGGING	= 8,
+	NET_KHTTPD_SERVERPORT	= 9,
+	NET_KHTTPD_DYNAMICSTRING= 10,
+	NET_KHTTPD_SLOPPYMIME   = 11,
+	NET_KHTTPD_THREADS	= 12,
+	NET_KHTTPD_MAXCONNECT	= 13
+};
+
+/* /proc/sys/net/sctp */
+enum {
+	NET_SCTP_RTO_INITIAL = 1,
+	NET_SCTP_RTO_MIN     = 2,
+	NET_SCTP_RTO_MAX     = 3,
+	NET_SCTP_RTO_ALPHA   = 4,
+	NET_SCTP_RTO_BETA    = 5,
+	NET_SCTP_VALID_COOKIE_LIFE       =  6,
+	NET_SCTP_ASSOCIATION_MAX_RETRANS =  7,
+	NET_SCTP_PATH_MAX_RETRANS        =  8,
+	NET_SCTP_MAX_INIT_RETRANSMITS    =  9,
+	NET_SCTP_HB_INTERVAL             = 10,
+	NET_SCTP_PRESERVE_ENABLE         = 11,
+	NET_SCTP_MAX_BURST               = 12,
+	NET_SCTP_ADDIP_ENABLE		 = 13,
+	NET_SCTP_PRSCTP_ENABLE		 = 14
+};
+
+/* /proc/sys/net/bridge */
+enum {
+	NET_BRIDGE_NF_CALL_ARPTABLES = 1,
+	NET_BRIDGE_NF_CALL_IPTABLES = 2,
+	NET_BRIDGE_NF_CALL_IP6TABLES = 3,
+	NET_BRIDGE_NF_FILTER_VLAN_TAGGED = 4
+};
+
 /* CTL_PROC names: */
 
 /* CTL_FS names: */
@@ -632,7 +670,7 @@ enum
 	FS_STATINODE=2,
 	FS_MAXINODE=3,	/* int:maximum number of inodes that can be allocated */
 	FS_NRDQUOT=4,	/* int:current number of allocated dquots */
-	/* was FS_MAXDQUOT */
+	FS_MAXDQUOT=5,	/* int:maximum number of dquots that can be allocated */
 	FS_NRFILE=6,	/* int:current number of allocated filedescriptors */
 	FS_MAXFILE=7,	/* int:maximum number of filedescriptors that can be allocated */
 	FS_DENTRY=8,
@@ -643,8 +681,10 @@ enum
 	FS_LEASES=13,	/* int: leases enabled */
 	FS_DIR_NOTIFY=14,	/* int: directory notification enabled */
 	FS_LEASE_TIME=15,	/* int: maximum time to wait for a lease break */
-	FS_DQSTATS=16,	/* dir: disc quota usage statistics and settings */
+	FS_DQSTATS=16,	/* disc quota usage statistics */
 	FS_XFS=17,	/* struct: control xfs parameters */
+	FS_AIO_NR=18,	/* current system-wide number of aio requests */
+	FS_AIO_MAX_NR=19	/* system-wide maximum number of aio requests */
 };
 
 /* /proc/sys/fs/quota/ */
@@ -656,8 +696,7 @@ enum {
 	FS_DQ_CACHE_HITS = 5,
 	FS_DQ_ALLOCATED = 6,
 	FS_DQ_FREE = 7,
-	FS_DQ_SYNCS = 8,
-	FS_DQ_WARNINGS = 9,
+	FS_DQ_SYNCS = 8
 };
 
 /* CTL_DEBUG names: */
@@ -668,7 +707,8 @@ enum {
 	DEV_HWMON=2,
 	DEV_PARPORT=3,
 	DEV_RAID=4,
-	DEV_MAC_HID=5
+	DEV_MAC_HID=5,
+	DEV_SCSI=6
 };
 
 /* /proc/sys/dev/cdrom */
@@ -711,12 +751,12 @@ enum {
 
 /* /proc/sys/dev/parport/parport n/devices/ */
 enum {
-	DEV_PARPORT_DEVICES_ACTIVE=-3,
+	DEV_PARPORT_DEVICES_ACTIVE=-3
 };
 
 /* /proc/sys/dev/parport/parport n/devices/device n */
 enum {
-	DEV_PARPORT_DEVICE_TIMESLICE=1,
+	DEV_PARPORT_DEVICE_TIMESLICE=1
 };
 
 /* /proc/sys/dev/mac_hid */
@@ -729,6 +769,11 @@ enum {
 	DEV_MAC_HID_ADB_MOUSE_SENDS_KEYCODES=6
 };
 
+/* /proc/sys/dev/scsi */
+enum {
+	DEV_SCSI_LOGGING_LEVEL=1
+};
+
 /* /proc/sys/abi */
 enum
 {
@@ -737,120 +782,7 @@ enum
 	ABI_DEFHANDLER_LCALL7=3,/* default handler for procs using lcall7 */
 	ABI_DEFHANDLER_LIBCSO=4,/* default handler for an libc.so ELF interp */
 	ABI_TRACE=5,		/* tracing flags */
-	ABI_FAKE_UTSNAME=6,	/* fake target utsname information */
+	ABI_FAKE_UTSNAME=6	/* fake target utsname information */
 };
 
-#ifdef __KERNEL__
-
-extern asmlinkage long sys_sysctl(struct __sysctl_args *);
-extern void sysctl_init(void);
-
-typedef struct ctl_table ctl_table;
-
-typedef int ctl_handler (ctl_table *table, int *name, int nlen,
-			 void *oldval, size_t *oldlenp,
-			 void *newval, size_t newlen, 
-			 void **context);
-
-typedef int proc_handler (ctl_table *ctl, int write, struct file * filp,
-			  void *buffer, size_t *lenp);
-
-extern int proc_dostring(ctl_table *, int, struct file *,
-			 void *, size_t *);
-extern int proc_dointvec(ctl_table *, int, struct file *,
-			 void *, size_t *);
-extern int proc_dointvec_bset(ctl_table *, int, struct file *,
-			      void *, size_t *);
-extern int proc_dointvec_minmax(ctl_table *, int, struct file *,
-				void *, size_t *);
-extern int proc_dointvec_jiffies(ctl_table *, int, struct file *,
-				 void *, size_t *);
-extern int proc_doulongvec_minmax(ctl_table *, int, struct file *,
-				  void *, size_t *);
-extern int proc_doulongvec_ms_jiffies_minmax(ctl_table *table, int,
-				      struct file *, void *, size_t *);
-
-extern int do_sysctl (int *name, int nlen,
-		      void *oldval, size_t *oldlenp,
-		      void *newval, size_t newlen);
-
-extern int do_sysctl_strategy (ctl_table *table, 
-			       int *name, int nlen,
-			       void *oldval, size_t *oldlenp,
-			       void *newval, size_t newlen, void ** context);
-
-extern ctl_handler sysctl_string;
-extern ctl_handler sysctl_intvec;
-extern ctl_handler sysctl_jiffies;
-
-
-/*
- * Register a set of sysctl names by calling register_sysctl_table
- * with an initialised array of ctl_table's.  An entry with zero
- * ctl_name terminates the table.  table->de will be set up by the
- * registration and need not be initialised in advance.
- *
- * sysctl names can be mirrored automatically under /proc/sys.  The
- * procname supplied controls /proc naming.
- *
- * The table's mode will be honoured both for sys_sysctl(2) and
- * proc-fs access.
- *
- * Leaf nodes in the sysctl tree will be represented by a single file
- * under /proc; non-leaf nodes will be represented by directories.  A
- * null procname disables /proc mirroring at this node.
- * 
- * sysctl(2) can automatically manage read and write requests through
- * the sysctl table.  The data and maxlen fields of the ctl_table
- * struct enable minimal validation of the values being written to be
- * performed, and the mode field allows minimal authentication.
- * 
- * More sophisticated management can be enabled by the provision of a
- * strategy routine with the table entry.  This will be called before
- * any automatic read or write of the data is performed.
- * 
- * The strategy routine may return:
- * <0: Error occurred (error is passed to user process)
- * 0:  OK - proceed with automatic read or write.
- * >0: OK - read or write has been done by the strategy routine, so 
- *     return immediately.
- * 
- * There must be a proc_handler routine for any terminal nodes
- * mirrored under /proc/sys (non-terminals are handled by a built-in
- * directory handler).  Several default handlers are available to
- * cover common cases.
- */
-
-/* A sysctl table is an array of struct ctl_table: */
-struct ctl_table 
-{
-	int ctl_name;			/* Binary ID */
-	const char *procname;		/* Text ID for /proc/sys, or zero */
-	void *data;
-	int maxlen;
-	mode_t mode;
-	ctl_table *child;
-	proc_handler *proc_handler;	/* Callback for text formatting */
-	ctl_handler *strategy;		/* Callback function for all r/w */
-	struct proc_dir_entry *de;	/* /proc control block */
-	void *extra1;
-	void *extra2;
-};
-
-/* struct ctl_table_header is used to maintain dynamic lists of
-   ctl_table trees. */
-struct ctl_table_header
-{
-	ctl_table *ctl_table;
-	struct list_head ctl_entry;
-};
-
-struct ctl_table_header * register_sysctl_table(ctl_table * table, 
-						int insert_at_head);
-void unregister_sysctl_table(struct ctl_table_header * table);
-
-#else /* __KERNEL__ */
-
-#endif /* __KERNEL__ */
-
 #endif /* _LINUX_SYSCTL_H */
